import typer
from . import portals
from . import transformations

app = typer.Typer()
app.add_typer(portals.app, name="portals")
app.add_typer(transformations.app, name="transformations")

@app.callback()
def callback():
    """
    Transform data into hardware-agnostic AI models, using the Edge Impulse API
    """

if __name__ == "__main__":
    app()


import requests
from ..common import api

def download_folder(jwt, portal_id, path, token, folder_path=''):
    # Get list of files portals/{portalId}/files
    # prefix gives the folder path to download from
    response = api.do_jwt_post(jwt,
                              "portals/%s/files" % portal_id,
                              {"prefix": folder_path + "/"},
                              portal_token=token)
    for item in response["files"]:
        # If a folder (size == 0) then recurse down into next subfolder and download
        if item['size'] == 0:
            # print("\tSubfolder '%s' found, starting recursion..." %
            #       item['name'])
            download_folder(jwt, portal_id, path, token, item['path'] + item['name'])
            continue
        # Get the download URL per file
        filepath = os.path.join(item["path"], item["name"])
        print("- %s..." % filepath, end="", flush=True)
        response = api.do_jwt_post(jwt,
                                  "portals/%s/files/download" % portal_id,
                                  {"path": filepath},
                                  portal_token=token)
        response = requests.get(response["url"])
        # Make sure os.path.join doesn't think filepath is a root path
        if filepath.startswith("/"):
            filepath = filepath.strip("/")
        dest = os.path.join(path, filepath)
        # Make sure the directory we are saving to exists
        os.makedirs(os.path.dirname(dest), exist_ok=True)
        open(dest, "wb").write(response.content)
        print(" downloaded to %s" % dest, flush=True)
    return




import requests
import json
from pprint import pprint

EI_API_URL = "https://studio.edgeimpulse.com/v1/api/"
EI_API_LOGIN_URL = "https://studio.edgeimpulse.com/v1/api-login"


def get_headers(org_key):
    return {
        "Accept": "application/json",
        "Content-Type": "application/json",
        "x-api-key": org_key
    }


def get_jwt_headers(jwt_token, portal_token=""):
    if portal_token == "":
        return {
            "Accept": "application/json",
            "Content-Type": "application/json",
            "x-jwt-token": jwt_token
        }
    else:
        return {
            "Accept": "application/json",
            "Content-Type": "application/json",
            "x-jwt-token": jwt_token,
            "x-token": portal_token
        }


def get_org_url(org_id):
    return EI_API_URL + "organizations/" + str(org_id)


def check_response(res, type="json", debug=False):
    payload = json.loads(res.text)
    if debug:
        print("check_response: ", end="")
        pprint(payload)
    if res.ok:
        if payload["success"]:
            return payload
    raise RuntimeError("Invalid response %s" % payload)


def do_org_get(org_id, org_key, endpoint, debug=False):
    url = (get_org_url(org_id) + endpoint)
    if debug:
        print("do_org_get: %s" % url)
    res = requests.get(url, headers=get_headers(org_key))
    return check_response(res, debug=debug)


def do_jwt_get(token, endpoint, portal_token="", debug=False):
    url = EI_API_URL + endpoint
    if debug:
        print("do_jwt_get: %s" % url)
    res = requests.get(url, headers=get_jwt_headers(token, portal_token))
    return check_response(res, debug=debug)


def do_jwt_post(token, endpoint, payload, portal_token="", debug=False):
    url = EI_API_URL + endpoint
    if debug:
        print("do_jwt_post: %s" % url)
    res = requests.post(url,
                        headers=get_jwt_headers(token, portal_token),
                        json=payload)
    return check_response(res, debug=debug)


def do_org_delete(org_id, org_key, endpoint, debug=False):
    url = (get_org_url(org_id) + endpoint)
    if debug:
        print("do_org_delete: %s" % url)
    res = requests.delete(url, headers=get_headers(org_key))
    return check_response(res, debug=debug)


def do_org_post(org_id, org_key, endpoint, payload, debug=False):
    url = (get_org_url(org_id) + endpoint)
    if debug:
        print("do_org_post: %s" % url)
    res = requests.post(url, headers=get_headers(org_key), json=payload)
    return check_response(res, debug=debug)


def get_jwt(username, password, debug=False):
    url = EI_API_LOGIN_URL
    payload = {"username": username, "password": password}
    if debug:
        print("get_jwt: %s" % url)
    res = requests.post(url,
                        headers={"Content-Type": "application/json"},
                        json=payload)
    return check_response(res, debug=debug)



import typer
import pandas as pd
from .common import api
from .common import opts

app = typer.Typer()


@app.callback()
def callback():
    """
    Manage organization data transformations on Edge Impulse
    """


@app.command()
def create(org_id: int = opts.REQUIRED_ORG_ID,
           org_key: str = opts.REQUIRED_ORG_KEY,
           container: str = typer.Option(..., help="the URI of the Docker container to run (may include version tag)"),
           name: str = typer.Option(None, help="the name of the transformation block"),
           description: str = typer.Option(None, help="a summary describing the transformation block")):
    """
    Create a data transformation block on Edge Impulse
    
    Organization portals and buckets are mounted as /portals/<id> and /buckets/<id> respectively.
    """
    # Use container as name if name is missing
    if name == None:
        name = container

    # Use name as description if description is missing
    if description == None:
        description = name

    # Define mountpoints
    # Mount portals
    response = api.do_org_get(org_id, org_key, "/portals")
    portals = response["portals"]
    mount_points = []
    for portal in portals:
        # keys: { "id", "name", "description",
        #         "url", "bucketName", "bucketPath",
        #         "bucketUrl", "created" }
        mount_points.append({
            "mountPoint": "/portals/%s" % portal["id"],
            "portalId": int(portal["id"]),
            "type": "portal"
        })
    # Mount buckets
    response = api.do_org_get(org_id, org_key, "/buckets")
    buckets = response["buckets"]
    for bucket in buckets:
        mount_points.append({
            "mountPoint": "/buckets/%s" % bucket["id"],
            "bucketId": int(bucket["id"]),
            "type": "bucket"
        })
    # No metadata, no arguments
    payload = {
        "allowExtraCliArguments": True,
        "cliArguments": "",
        "description": description,
        "dockerContainer": container,
        "indMetadata": False,
        "name": name,
        "operatesOn": "standalone",
        "additionalMountPoints": mount_points
    }
    response = api.do_org_post(org_id, org_key, "/transformation",
                              payload)
    print(response)

@app.command()
def list(org_id: int = opts.REQUIRED_ORG_ID,
         org_key: str = opts.REQUIRED_ORG_KEY):
    """
    List data transformation blocks for an Edge Impulse organization
    """
    response = api.do_org_get(org_id, org_key, "/transformation")
    blocks = response["transformationBlocks"]
    if len(blocks) > 0:
        df = pd.DataFrame(blocks)
        # df will have the following columns:
        # print(df.columns.values)
        # [
        #     'id'
        #     'name'
        #     'dockerContainer'
        #     'dockerContainerManagedByEdgeImpulse'
        #     'created'
        #     'description'
        #     'cliArguments'
        #     'indMetadata'
        #     'additionalMountPoints'
        #     'operatesOn'
        #     'allowExtraCliArguments'
        # ]
        df = df[["id", "name", "dockerContainer", "additionalMountPoints"]]
        print(df.to_string(index=False))
    else:
        print("This organization has no data transformation blocks")

@app.command()
def delete(org_id: int = opts.REQUIRED_ORG_ID,
           org_key: str = opts.REQUIRED_ORG_KEY,
           transformation_id = typer.Option(..., help="the ID of the data transformation to delete")):
    """
    Delete a data transformation block for an Edge Impulse organization
    """
    response = api.do_org_delete(org_id, org_key,
                                "/transformation/%s" % id)
    print(response)



import typer

HELP_PORTAL_ID = "the ID of the portal to download files from"
REQUIRED_ORG_ID = typer.Option(..., help="the ID of the Edge Impulse organization that the portal belongs to", envvar="EI_ORG_ID")
REQUIRED_ORG_KEY = typer.Option(..., help="an Edge Impulse Organization API key", envvar="EI_ORG_KEY")
REQUIRED_EI_USERNAME = typer.Option(..., help="the Edge Impulse username to login with", envvar="EI_USERNAME")
REQUIRED_EI_PASSWORD = typer.Option(..., help="the Edge Impulse user password", envvar="EI_PASSWORD")
REQUIRED_PORTAL_ID = typer.Option(..., help=HELP_PORTAL_ID)
OPTIONAL_PORTAL_ID = typer.Option(None, help=HELP_PORTAL_ID)



import typer
import pandas as pd
from .common import api
from .common import utils
from .common import opts

app = typer.Typer()

@app.callback()
def callback():
    """
    Work with Edge Impulse Upload Portals
    """

@app.command()
def create(org_id: int = opts.REQUIRED_ORG_ID,
           org_key: str = opts.REQUIRED_ORG_KEY,
           name: str = typer.Option(..., help="the name of the portal to create")):
    """
    Create an organization data upload portal on Edge Impulse
    """
    # print(org_id, name)
    # Use the name as the bucket path (just replace spaces with dashes)
    payload = {
        "name": name,
        "bucketId": 0,
        "bucketPath": name.replace(" ", "-").replace("\\",
                                                     "-").replace("/", "-")
    }
    api.do_org_post(org_id, org_key, "/portals/create", payload, debug=False)

@app.command()
def show(org_id: int = opts.REQUIRED_ORG_ID,
         org_key: str = opts.REQUIRED_ORG_KEY,
         portal_id: int = opts.OPTIONAL_PORTAL_ID,
         contents: bool = typer.Option(False, help="Show the contents of the portal")):
    """
    Show upload portal details of an Edge Impulse organization

    If --portal-id is provided, only the details for that portal will be shown
    """
    if portal_id == None or isinstance(portal_id, typer.models.OptionInfo):
        response = api.do_org_get(org_id, org_key, "/portals")
        portals = response["portals"]
        if len(portals) > 0:
            df = pd.DataFrame(portals)
            # df will have the following columns:
            # print(df.columns.values)
            # [
            #     "id"
            #     "name"
            #     "description"
            #     "url"
            #     "bucketName"  👈🏽 Irrelevant to customers
            #     "bucketPath"  👈🏽 Irrelevant to customers
            #     "bucketUrl"   👈🏽 Irrelevant to customers
            #     "created"
            # ]
            output = []
            for portal in portals:
                del portal["bucketName"]
                del portal["bucketPath"]
                del portal["bucketUrl"]
                output.append(portal)
            print(df[["id", "name", "url"]].to_string(index=False))
            return output
        else:
            print("This organization has no portals")
            return None
    else:
        response = api.do_org_get(org_id, org_key, "/portals/%s" % portal_id)
        portal = response
        del portal["success"]
        if contents:
            portal["files"] = response["files"]
            response = api.do_org_get(org_id, org_key, "/portals/%s/verify" % portal_id)
            print("Listing the contents of portal %s" % portal_id)
            for item in response["files"]:
                print("- %s/%s" % (item["folderName"], item["name"]))
        return portal

@app.command()
def download(org_id: int = opts.REQUIRED_ORG_ID,
             username: str = opts.REQUIRED_EI_USERNAME,
             password: str = opts.REQUIRED_EI_PASSWORD,
             portal_id: int = opts.REQUIRED_PORTAL_ID,
             path: str = typer.Option(..., help="the path where the portal files should be downloaded")):
    """
    Download the contents of an Edge Impulse organization portal
    """
    # Get JWT token
    response = api.get_jwt(username, password)
    jwt = response["token"]
    # Get portal token
    response = api.do_jwt_get(
        jwt, "organizations/%s/portals/%s" % (org_id, portal_id))
    token = response["token"]
    # Download files in base folder
    print("Downloading the contents of portal %s" % portal_id, flush=True)
    utils.download_folder(jwt, portal_id, path, token, folder_path='')

@app.command()
def delete(org_id: int = opts.REQUIRED_ORG_ID,
           org_key: str = opts.REQUIRED_ORG_KEY,
           portal_id: int = opts.REQUIRED_PORTAL_ID):
    """
    Delete an organization portal from Edge Impulse
    """
    response = api.do_org_delete(org_id, org_key,
                                "/portals/%s/delete" % portal_id)
    if response["success"]:
        msg = "Portal %s successfully deleted" % portal_id
    else:
        msg = "Could not delete portal %s" % portal_id
    print(msg)
    response["message"] = msg
    return response

if __name__ == "__main__":
    app()


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['edge_ai', 'edge_ai.common']

package_data = \
{'': ['*']}

install_requires = \
['pandas>=1.5.3,<2.0.0', 'requests>=2.28.2,<3.0.0', 'typer[all]>=0.7.0,<0.8.0']

entry_points = \
{'console_scripts': ['ei = edge_ai.main:app']}

setup_kwargs = {
    'name': 'edge-ai',
    'version': '0.1.2',
    'description': '',
    'long_description': '# edge-ai\n\n`edge-ai` is a python module and CLI to transform data into hardware-agnostic AI models, using the Edge Impulse API.\n\n## Installation\n\n```console\npip install edge-ai\n```\n\n## CLI Usage\n\n### `ei`\n\nTransform data into hardware-agnostic AI models, using the Edge Impulse API\n\n**Usage**:\n\n```console\n$ ei [OPTIONS] COMMAND [ARGS]...\n```\n\n**Options**:\n\n* `--install-completion`: Install completion for the current shell.\n* `--show-completion`: Show completion for the current shell, to copy it or customize the installation.\n* `--help`: Show this message and exit.\n\n**Commands**:\n\n* `portals`: Work with Edge Impulse Upload Portals\n* `transformations`: Manage organization data transformations...\n\n#### `ei portals`\n\nWork with Edge Impulse Upload Portals\n\n**Usage**:\n\n```console\n$ ei portals [OPTIONS] COMMAND [ARGS]...\n```\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n**Commands**:\n\n* `create`: Create an organization data upload portal...\n* `delete`: Delete an organization portal from Edge...\n* `download`: Download the contents of an Edge Impulse...\n* `show`: Show upload portal details of an Edge...\n\n##### `ei portals create`\n\nCreate an organization data upload portal on Edge Impulse\n\n**Usage**:\n\n```console\n$ ei portals create [OPTIONS]\n```\n\n**Options**:\n\n* `--org-id INTEGER`: the ID of the Edge Impulse organization that the portal belongs to  [env var: EI_ORG_ID; required]\n* `--org-key TEXT`: an Edge Impulse Organization API key  [env var: EI_ORG_KEY; required]\n* `--name TEXT`: the name of the portal to create  [required]\n* `--help`: Show this message and exit.\n\n##### `ei portals delete`\n\nDelete an organization portal from Edge Impulse\n\n**Usage**:\n\n```console\n$ ei portals delete [OPTIONS]\n```\n\n**Options**:\n\n* `--org-id INTEGER`: the ID of the Edge Impulse organization that the portal belongs to  [env var: EI_ORG_ID; required]\n* `--org-key TEXT`: an Edge Impulse Organization API key  [env var: EI_ORG_KEY; required]\n* `--portal-id INTEGER`: the ID of the portal to download files from  [required]\n* `--help`: Show this message and exit.\n\n##### `ei portals download`\n\nDownload the contents of an Edge Impulse organization portal\n\n**Usage**:\n\n```console\n$ ei portals download [OPTIONS]\n```\n\n**Options**:\n\n* `--org-id INTEGER`: the ID of the Edge Impulse organization that the portal belongs to  [env var: EI_ORG_ID; required]\n* `--username TEXT`: the Edge Impulse username to login with  [env var: EI_USERNAME; required]\n* `--password TEXT`: the Edge Impulse user password  [env var: EI_PASSWORD; required]\n* `--portal-id INTEGER`: the ID of the portal to download files from  [required]\n* `--path TEXT`: the path where the portal files should be downloaded  [required]\n* `--help`: Show this message and exit.\n\n##### `ei portals show`\n\nShow upload portal details of an Edge Impulse organization\n\nIf --portal-id is provided, only the details for that portal will be shown\n\n**Usage**:\n\n```console\n$ ei portals show [OPTIONS]\n```\n\n**Options**:\n\n* `--org-id INTEGER`: the ID of the Edge Impulse organization that the portal belongs to  [env var: EI_ORG_ID; required]\n* `--org-key TEXT`: an Edge Impulse Organization API key  [env var: EI_ORG_KEY; required]\n* `--portal-id INTEGER`: the ID of the portal to download files from\n* `--contents / --no-contents`: Show the contents of the portal  [default: no-contents]\n* `--help`: Show this message and exit.\n\n#### `ei transformations`\n\nManage organization data transformations on Edge Impulse\n\n**Usage**:\n\n```console\n$ ei transformations [OPTIONS] COMMAND [ARGS]...\n```\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n**Commands**:\n\n* `create`: Create a data transformation block on Edge...\n* `delete`: Delete a data transformation block for an...\n* `list`: List data transformation blocks for an...\n\n##### `ei transformations create`\n\nCreate a data transformation block on Edge Impulse\n\nOrganization portals and buckets are mounted as /portals/<id> and /buckets/<id> respectively.\n\n**Usage**:\n\n```console\n$ ei transformations create [OPTIONS]\n```\n\n**Options**:\n\n* `--org-id INTEGER`: the ID of the Edge Impulse organization that the portal belongs to  [env var: EI_ORG_ID; required]\n* `--org-key TEXT`: an Edge Impulse Organization API key  [env var: EI_ORG_KEY; required]\n* `--container TEXT`: the URI of the Docker container to run (may include version tag)  [required]\n* `--name TEXT`: the name of the transformation block\n* `--description TEXT`: a summary describing the transformation block\n* `--help`: Show this message and exit.\n\n##### `ei transformations delete`\n\nDelete a data transformation block for an Edge Impulse organization\n\n**Usage**:\n\n```console\n$ ei transformations delete [OPTIONS]\n```\n\n**Options**:\n\n* `--org-id INTEGER`: the ID of the Edge Impulse organization that the portal belongs to  [env var: EI_ORG_ID; required]\n* `--org-key TEXT`: an Edge Impulse Organization API key  [env var: EI_ORG_KEY; required]\n* `--transformation-id TEXT`: the ID of the data transformation to delete  [required]\n* `--help`: Show this message and exit.\n\n##### `ei transformations list`\n\nList data transformation blocks for an Edge Impulse organization\n\n**Usage**:\n\n```console\n$ ei transformations list [OPTIONS]\n```\n\n**Options**:\n\n* `--org-id INTEGER`: the ID of the Edge Impulse organization that the portal belongs to  [env var: EI_ORG_ID; required]\n* `--org-key TEXT`: an Edge Impulse Organization API key  [env var: EI_ORG_KEY; required]\n* `--help`: Show this message and exit.\n',
    'author': 'Jorge Silva',
    'author_email': 'jorge@edgeimpulse.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
